/*
---------------------------------------------------------------------------
 Algo10-3.c   C program for implementing Algorithm 10.3
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 10.3 (Crank-Nicholson Method for the Heat Equation).
 Section   10.2, Parabolic Equations, Page 517
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 10.3 (Crank-Nicholson Method for the Heat Equation)
                                              2
 To approximate the solution of u_(t)(x,t) = c  *u_(xx)(x,t) 

 over R = {(x,t): 0 <= x <= a, 0 <= t <= b} with 

 u(x,0) = f(x)  for  0 <= x <= a  and 

 u(0,t) = c_1,  u(a,t) = c_2  for  0 <= t <= b. 

 User has to supply a function Fi. An Example is included
 in this program. See also in the book on page 515.

---------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define Max 50

/* Global Variables */

double H;                /* Step size  */

/* Prototypes */

double Fi(int i);

/* Grid function for amplitude */

double Fi(int i)
{
    extern double H;
    double arg;
    double Pi = 3.1415926535;

    arg = H * ( i - 1.0 );

    if( (arg >= 0) && (arg <= 1.0) ) 
    {
        return (  sin( Pi * arg ) + sin( 3.0 * Pi * arg ) );
    }
    else
    {
        printf(" Fi() : Argument not in range ! Exiting !\n");
        printf(" arg  : %lf\n", arg);
        exit(0);
    }
        
    return ;
}

/* -------------------------------------------------------- */

/*  Main program for algorithm 10-3  */


/*  In  C  the fields begin with index 0. 
    In this program it is pretended that they begin with 1.
    So be careful with U[i][j] with i or j = 0.  */

void main(void)

{
    int i, j;                /* Loop Counters                         */
    double A, B;             /* INPUT : Width and height of Rectangle */
    int N, M;                /* dimensions of the grid                */
    double U[Max][Max];      /* Grid-Amplitudes                       */
    double C, C1, C2;        /* constants                             */
    double K, R, S1, S2;     /* constants                             */
    double Va[Max], Vb[Max]; /* diagonal and off-diagonal             */
    double Vc[Max], Vd[Max]; /* elements of matrix A (see book)       */
    double A0[Max], B0[Max]; /* for replacements fo the Va, Vb, Vc    */
    double C0[Max], D0[Max]; /* and Vd                                */
    double X0[Max];          /* Solution vector for tridiagonal syst. */
    double T;


    printf("----- Crank-Nicholson Method for the Heat Equation ----\n");
    printf("--------- Try Example 10.4 on page 515 ----------------\n");
    printf("-------------------------------------------------------\n");

    printf("Please enter A (interval boundary of x)\n");
    printf("For present example enter  1 :\n");
    scanf("%lf",&A);
    printf("Please enter B (interval boundary of t)\n");
    printf("For present example enter  0.1 :\n");
    scanf("%lf",&B);
    printf("You entered A = %lf and B = %lf \n", A, B);
    printf("------------------------------------------------\n");
    printf("Please enter Constant C from HEAT Equation :\n");
    printf("For present example enter  1 :\n");
    scanf("%lf",&C);
    printf("You entered C = %lf\n", C);
    printf("------------------------------------------------\n");
    printf("Please enter Constant C1 = u(0,t) :\n");
    printf("For present example enter  0 :\n");
    scanf("%lf",&C1);
    printf("You entered C1 = %lf\n", C1);
    printf("------------------------------------------------\n");
    printf("Please enter Constant C2 = u(a,t) :\n");
    printf("For present example enter  0 :\n");
    scanf("%lf",&C2);
    printf("You entered C2 = %lf\n", C2);
    printf("------------------------------------------------\n");
    printf("\n");
    printf("Please enter dimension of grid in x-direction :\n");
    printf("For present example enter  11 :\n");
    scanf("%d",&N);
    printf("Please enter dimension of grid in y-direction :\n");
    printf("For present example enter  11 :\n");
    scanf("%d",&M);
    printf("You entered N = %d and M = %d\n", N, M);
    printf("------------------------------------------------\n");
    printf("\n");

    /* Only to make the 0-indexes save we initialize them to zero.
       This is only because in C fields begin with index 0. 
       In this program we ignore the index 0. */

    for ( i = 0; i < N; i++ ) U[i][0] = 0.0;
    for ( i = 1; i < M; i++ ) U[0][i] = 0.0;

    /* Compute step sizes */

    H  = A / ( N - 1 );

    /* Compute ratios and constants */

    K  = B / ( M - 1 );
    R  = C * C * K / ( H * H );
    S1 = 2.0 + 2.0 / R;
    S2 = 2.0 / R  - 2.0;

    /* Boundary conditions */

    for ( j = 1; j <= M; j++ ) 
    {
           U[1][j] = C1;
           U[N][j] = C2;
    }

    /* First row  */

    for ( i = 2; i <= N - 1 ; i++ ) U[i][1] = Fi(i);

    /* Form the diagonal elements of the matrix A 
       (see chapter 10.2 in book for explanations) */

    for ( i = 1; i <= N ; i++ ) Vd[i] = S1;
    Vd[1] = 1.0;
    Vd[N] = 1.0;

    /* Form the off diagonal Elements of the matrix A */

    for ( i = 1; i <= N - 1 ; i++ ) 
    {
        Va[i] = -1.0;
        Vc[i] = -1.0;
    }
       
    Va[N-1] = 0.0;
    Vc[1]   = 0.0;

    Vb[1] = C1;
    Vb[N] = C2;

    /* Construct successive rows in the grid */

    for ( j = 2; j <= M; j++ )
    {
        for ( i = 2; i <= N - 1; i++ ) 
        {
            Vb[i] = U[i-1][j-1] + U[i+1][j-1] + S2 * U[i][j-1];
        }

        /* Do what subroutine TriSystem in the book does */
        /* Solve the dridiagonal system. */

        /* Vector replacement first */

        for ( i = 1; i <= N; i++ ) 
        {
            A0[i] = Va[i];
            B0[i] = Vb[i];
            C0[i] = Vc[i];
            D0[i] = Vd[i];
        }

        for ( i = 2; i <= N; i++ ) 
        {
            T     = A0[i-1] / D0[i-1];
            D0[i] = D0[i] - T * C0[i-1];
            B0[i] = B0[i] - T * B0[i-1];
        }

        X0[N] = B0[N] / D0[N];

     for ( i = N - 1; i >= 1; i-- ) X0[i] = (B0[i] - C0[i]*X0[i+1]) / D0[i];

    /* Solution vector is now available */

    /* Next row in grid */

        for ( i = 1; i <= N; i++ ) U[i][j] = X0[i];

    } /* End of outer for-loop in main */

    /* Output the solution */

    for ( j = 1; j <= M; j++ )
    {
        printf("%8.6lf ", K * (j - 1) );
        for ( i = 1; i <= N; i++ ) printf("%8.6lf ", U[i][j]);
        printf("\n");
        if ( j < M )
        { 
            printf("Press enter to see the next row.\n");
            scanf ("%c", i);
        }
    }

}   /* End of main program */

