/*
---------------------------------------------------------------------------
 Algo7-6.c   C program for implementing Algorithm 7.6
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 7.6 (Gauss-Legendre Quadrature).
 Section   7.5, Gauss-Legendre Integration, Page 397
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------

 Algorithm 7.6 (Gauss-Legendre Quadrature).  To  approximate the integral

     b
     /              B - A
     | f(x)dx   =   ----- [ w_(N,1) * f(t_(N,1)) + w_(N,2) * f(t_(N,2))
     /                2
     a                      +...+ w_(N,N) * f(t_(N,N)) ]

 by sampling f(x) at the N unequally spaced points 
 t_(N,1), t_(N,2),...,t_(N,N).  The change of variable

        a+b     b-a                b-a
    t = ---  +  ---*x    and  dt = ---*dx
         2       2                  2

 are used. The abscissas x_(N,1), x_(N,2), .... , x(N,N)  and the
 corresponding weights   w_(N,1), w_(N,2), .... , w(N,N)  must be
 obtained from a table of known values.

---------------------------------------------------------------------------
*/

/* User has to supply a function named : ffunction
   An example is included in this program */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>


/*  define prototype for USER-SUPPLIED function f(x)  */

    double ffunction(double x);


/*  EXAMPLE for "ffunction" . See page 395 example 7.18  */

    double ffunction(double x)

    {
        return ( 5.0 * pow(x,4) );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 7.6  */

    void main()

{
    int K;                     /* loop counters                  */
    int N;                     /* INPUT : number of quadrature
                                          points  */
    double A,B;                /* INPUT : boundaries of integral */
    double T;                  /* Argument                       */
    double SUM     = 0;        /* Aprox. integral value          */
    double x[8];               /* Abscissas                      */
    double w[8];               /* Weights                        */

    printf("--------Example 7.18----page 395--------------\n");
    printf("Please enter the boundaries of the integral [A,B]\n");
    printf("EXAMPLE: A = -1 and B = 1, so type: -1  1\n");
    scanf("%lf %lf",&A, &B);
    printf("The boundaries of the integral are: %lf %lf\n",A, B);
    printf("----------------------------------------------\n");
    printf("Number of quadrature points  ? [2,3,4,5,6,7 or 8 ]\n");
    scanf("%d",&N);

    switch (N)   /* select the points and weights according to N */
    {
     case 2:
       x[0] = -0.5773502692;
       x[1] =  0.5773502692;
       w[0] =  1.0;
       w[1] =  1.0;
       break;
     case 3:
       x[0] = 0.7745966692;
       x[1] =-0.7745966692;
       x[2] = 0.0;
       w[0] = 0.5555555556;
       w[1] = 0.5555555556;
       w[2] = 0.8888888888;
       break;
     case 4:
       x[0] = 0.8611363116;
       x[1] =-0.8611363116;
       x[2] = 0.3399810436;
       x[3] =-0.3399810436;
       w[0] = 0.3478548451;
       w[1] = 0.3478548451;
       w[2] = 0.6521451549;
       w[3] = 0.6521451549;
       break;
     case 5:
       x[0] = 0.9061798459;
       x[1] =-0.9061798459;
       x[2] = 0.5384693101;
       x[3] =-0.5384693101;
       x[4] = 0.0000000000;
       w[0] = 0.2369268851;
       w[1] = 0.2369268851;
       w[2] = 0.4786286705;
       w[3] = 0.4786286705;
       w[4] = 0.5688888888;
       break;
     case 6:
       x[0] = 0.9324695142;
       x[1] =-0.9324695142;
       x[2] = 0.6612093865;
       x[3] =-0.6612093865;
       x[4] = 0.2386191861;
       x[5] =-0.2386191861;
       w[0] = 0.1713244924;
       w[1] = 0.1713244924;
       w[2] = 0.3607615730;
       w[3] = 0.3607615730;
       w[4] = 0.4679139346;
       w[5] = 0.4679139346;
       break;
     case 7:
       x[0] = 0.9491079123;
       x[1] =-0.9491079123;
       x[2] = 0.7415311856;
       x[3] =-0.7415311856;
       x[4] = 0.4058451514;
       x[5] =-0.4058451514;
       x[6] = 0.0;
       w[0] = 0.1294849662;
       w[1] = 0.1294849662;
       w[2] = 0.2797053915;
       w[3] = 0.2797053915;
       w[4] = 0.3818300505;
       w[5] = 0.3818300505;
       w[6] = 0.4179591837;
       break;
     case 8:
       x[0] = 0.9602898565;
       x[1] =-0.9602898565;
       x[2] = 0.7966664774;
       x[3] =-0.7966664774;
       x[4] = 0.5255324099;
       x[5] =-0.5255324099;
       x[6] = 0.1834346425;
       x[7] =-0.1834346425;
       w[0] = 0.1012285363;
       w[1] = 0.1012285363;
       w[2] = 0.2223810345;
       w[3] = 0.2223810345;
       w[4] = 0.3137066459;
       w[5] = 0.3137066459;
       w[6] = 0.3626837834;
       w[7] = 0.3626837834;
       break;
       /* if unvalid N has been entered : */
     default: printf("Only N = 2,3,4,5,6,7 or 8 is valid as entry.");
     exit(1);
    }

     for (K = 1; K <= N; K++)
     {  /* remember fields in C language begin with element 0 */

        T = (A + B) / 2.0 + x[K-1] * (B - A) / 2.0;
        SUM += w[K-1] * ffunction(T);
     }
        SUM = SUM * (B - A) / 2.0;


    printf("------%d point Gauss-Legendre integration-------\n",N);

    printf(" The best approximation for value of the integral\n");
    printf(" using Gauss-Legendre integration is = %lf\n", SUM );

    printf("----------------------------------------------\n");

}   /* End of main program */

