/*
---------------------------------------------------------------------------
 Algo8-2.c   C program for implementing Algorithm 8.2
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 8.2 (Nelder-Mead's Minimization Method).
 Section   8.1, Minimization of a Function, Page 414
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 8-2 (Nelder-Mead's Minimization Method).

 To approximate a local minimum of  F(x_1, x_2, ... , x_N), where  F
 is a continuous function of  N  real variables, and given the
 N+1  initial starting points  V_k = (v_(k,1), ... , v_(k,N)) for
 k = 0,1,...,N.

 User has to supply the functions which is to be minimized:
 The function is named : ffunction 
 An example is included in this program.

---------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define  VarMax  30     /* Upper limit for number of variables */

/* Function Prototypes */

void Order(int N);
void Size(int N);
void Newpoints(int N);
void Shrink(int N);
void Improve(int N);

/*  Global Variables  */
    
double V[VarMax][VarMax];
double Y[VarMax], R[VarMax], M[VarMax];
int Lo, Li, Ho, Hi;
double Norm, YR;
double Farg[VarMax];   /* vector used as argument for ffunction */

/*  define prototype for USER-SUPPLIED function */

    double ffunction(void); 

/*  EXAMPLE for "ffunction" : see example 8.4 on page 409  */

    double ffunction(void)
    {
        return ( pow(Farg[1], 2.0 ) - 4.0 * Farg[1]
               + pow(Farg[2], 2.0) - Farg[2] - Farg[1] * Farg[2] );
    }

/* -------------------------------------------------------- */


/*  In  C  the fields begin with index 0. 
    In this program it is pretended that they begin with  1
    except for the matrix V where the rows start with 0.
    So be careful when addressing fields with index  0.  */


/*  Main program for algorithm 8.2  */

void main(void)
{
    int i,j;                   /* Loop counter                        */
    int N;                     /* Number of variables in ffunction    */
    int Min = 10;              /* Minimum number of iterations        */
    int Max = 200;             /* Maximum number of iterations        */
    int m = 0;                 /* While loop counter                  */
    double Tol = 1E-07;        /* Convergence criterion               */

    printf("---------Nelder-Mead Simplex-Method-------------\n");
    printf("--------- Example 8.4 on page 409 --------------\n");
    printf("------------------------------------------------\n");
    printf("Please enter number of variables :\n");
    printf("For used Example type  : 2\n");
    scanf("%d", &N);
    printf("You entered N = %d\n", N);
    if ( N >= VarMax )
    {
        printf("Too many variables ! Terminating ...\n");
        printf("(You can increase VarMax in the program if you want.)\n");
        exit(0);
    }
    printf("------------------------------------------------\n");

    /* Get N+1 initial points and their N coordinates */

    printf("In this example start with the vertexes :\n");
    printf("V_1 = (0,0)  V_2 = (1.2,0.0)  V_3 = (0.0,0.8) :\n");


    for (j = 0; j <= N; j++) 
    {
        for (i = 1; i <= N; i++) 
        {
            printf("Enter the %d coordinate for point no. %d\n", i, j+1);
            scanf("%lf", &V[j][i]);  
            printf(" You entered : %lf\n", V[j][i]);
        }
    }

    /* Compute function value at V_j */

    for (j = 0; j <= N; j++) 
    {
        for (i = 1; i <= N; i++) Farg[i] = V[j][i];
        Y[j] = ffunction();
    }

    Order(N);
  
printf("\n");

    while (  ( (Y[Hi] > (Y[Lo] + Tol)) && (m < Max) ) || (m < Min) )
     {
        Newpoints(N);
        Improve(N);
        Order(N);
        m++;                            /* Increment the counter */
     }

    Size(N);

    /* Output */

    printf("---------------------------------------------------\n");
    printf("Coordinates of local minimum are :\n");
    for ( i = 1; i <= N; i++) printf("%lf ", V[Lo][i]);
    printf("...................................................\n");
    printf("The function value at this point is : %lf\n", Y[Lo]);
    printf("The size of the final simplex is : %lf\n", Norm);
    printf("...................................................\n");

}   /* End of main programm */


/* Subroutine Order */

void Order(int N)
{
    int j;

    /* Find indices : Lo, best vertex; Hi, worst vertex */

    Lo = 0.0;
    Hi = 0.0;

    for ( j = 1; j <= N; j++ )
    {
        if (Y[j] < Y[Lo] ) Lo = j;
        if (Y[j] > Y[Hi] ) Hi = j;
    }

    Li = Hi;
    Ho = Lo;

    for ( j = 0; j <= N; j++ )
    {
        if ( (j != Lo) && (Y[j] < Y[Li] ) ) Li = j;
        if ( (j != Hi) && (Y[j] > Y[Ho] ) ) Ho = j;
    }

}  /*  End of subroutine Order */


/* Subroutine Size */

void Size(int N)
{
   /* Size of simplex */
 
    int i, j;
    double S;

    Norm = 0.0;    
    for ( j = 0; j <= N; j++ )
    {
        S = 0;
        for ( i = 1; i <= N; i++ )  S += pow( (V[Lo][i] - V[j][i]), 2.0 );
        if ( S > Norm ) Norm = S;
    }
    Norm = sqrt(Norm);

}   /* End of subroutine Size */

/* Subroutine Newpoints */

void Newpoints(int N)
{
    int k, j;
    double S;

    for ( k = 1; k <= N; k++ )
    {
        S = 0.0;
        for ( j = 0; j <= N; j++ ) S += V[j][k];
        M[k] = ( S - V[Hi][k] ) / N;
    }

    for ( k = 1; k <= N; k++ ) Farg[k] = R[k] = 2.0 * M[k] - V[Hi][k];
    YR = ffunction();

}   /* End of subroutine Newpoints */

/*  Subroutine Shrink  */

void Shrink(int N)
{
    int j, k;

    for ( j = 0; j <= N; j++ )
    {
        if( j != Lo ) 
        {
          for ( k = 1; k <= N; k++ ) Farg[k] = V[j][k] = ( V[j][k] + V[Lo][k] ) 
/ 2.0;
          Y[j] = ffunction();
        }
    }

}   /* End of subroutine Shrink */

/*  Subroutine Replace */

void Replace(int N)
{
    int k;

    /* Replace V with R */

    for ( k = 1; k <= N; k++ ) V[Hi][k] = R[k];
    Y[Hi] = YR;

}   /* End of subroutine Replace */


/*  Subroutine Improve */

void Improve(int N)
{
    int k;
    double E[VarMax], C[VarMax];
    double YE, YC;

    /* N-M : use Y[Lo], not Y[Li] */
    if ( YR < Y[Ho] )
    {
        if ( Y[Li] < YR ) Replace(N);   /* Replace V with R */
        else
        {
            /* Construct E */
            for (k = 1; k <= N; k++) Farg[k] = E[k] = 2.0 * R[k] - M[k];
            YE = ffunction();

            /* N-M: use Y[Lo], not Y[Li] */
            if ( YE < Y[Li] )
            {
               /* Replace V with E */
               for (k = 1; k <= N; k++) V[Hi][k] = E[k];
               Y[Hi] = YE;
            }
            else Replace(N);   /* Replave V with R */
        }        
    }
    else 
    {   
       if( YR < Y[Hi] ) Replace(N);  /* Replace V with R */  

       for (k = 1; k <= N; k++) Farg[k] = C[k] = ( V[Hi][k] + M[k] ) / 2.0;
       YC = ffunction();

       if( YC < Y[Hi] )
       {
           for (k = 1; k <= N; k++) V[Hi][k] = C[k]; /* Replace V with C */
           Y[Hi] = YC;
       }
       else Shrink(N);

    }  /* End of all conditions */

}   /* End of subroutine Improve */

