/*
---------------------------------------------------------------------------
 Algo9-10.c   C program for implementing Algorithm 9.10
 Algorithm translated to  C  by: Dr. Norman Fahrer
 IBM and Macintosh verification by: Daniel Mathews
 
 NUMERICAL METHODS: C Programs, (c) John H. Mathews 1995
 To accompany the text:
 NUMERICAL METHODS for Mathematics, Science and Engineering, 2nd Ed, 1992
 Prentice Hall, Englewood Cliffs, New Jersey, 07632, U.S.A.
 Prentice Hall, Inc.; USA, Canada, Mexico ISBN 0-13-624990-6
 Prentice Hall, International Editions:   ISBN 0-13-625047-5
 This free software is compliments of the author.
 E-mail address:       in%"mathews@fullerton.edu"
 
 Algorithm 9.10. (Finite-Difference Method).
 Section   9.9, Finite-Difference Method, Page 496
---------------------------------------------------------------------------
*/
/*
---------------------------------------------------------------------------
 
 Algorithm 9.10 (Finite-Difference Method)

 To approximate the solution of the boundary value problem

 x''  =  p(t)*x'(t) + q(t)*x(t) + r(t)  with  x(a) = alpha
  
 and  x(b) = beta  over the interval [a,b] by using the finite-difference

                     2
 method of order  O(h ).

 Remark : The mesh is  a = t_1 < ... < t_(N+1) = b  and the solution
                                    N+1
          points are  { (t_j, x_j) }      ;
                                    j = 1

 User has to supply  functions named : pffunction, qffunction and
 rffunction.
 
 An example is included in this program.

---------------------------------------------------------------------------
*/

#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define MAX 300


/*  define prototype for USER-SUPPLIED function f(x)  */

    double pffunction(double t);
    double qffunction(double t);
    double rffunction(double t);

/*  Example is : 9.18 on page 493

/*  EXAMPLE for "pffunction"   */


    double pffunction(double t)

    {
        return ( 2.0 * t/ ( 1 + t * t) );
    }

/*  EXAMPLE for "qffunction"   */

    double qffunction(double t)
                              
    {
        return ( - 2.0 / ( 1 + t * t) );
    }

/*  EXAMPLE for "rffunction"   */

    double rffunction(double t)

    {
        return ( 1.0 );
    }

/* -------------------------------------------------------- */

/*  Main program for algorithm 9.10  */

void main(void)

{
    int i, K, J;               /* Loop counter                   */
    int N;                     /* Number of steps < MAX          */
    double A, B;               /* Endpoints and inital value     */
    double H;                  /* Compute the step size          */
    double T[MAX]; 
    double Va[MAX], Vb[MAX], Vc[MAX], Vd[MAX], Vt[MAX];
    double X0[MAX], X[MAX], Temp;
    double Alpha, Beta;


    printf("----------Finite-Difference Method----------------\n");
    printf("--------- Example 9.18 on page 493 -------------\n");
    printf("-----------------------------------------------\n");
    printf("Please enter endpoints of the interval [A,B]:\n");
    printf("For used Example type  : 0, 4.0\n");
    scanf("%lf, %lf", &A, &B);
    printf("You entered [%lf, %lf]\n", A, B);
    printf("---------------------------------------------\n");
    printf("Please enter boundary values x(A) and x(B)\n");
    printf("For used Example type : 1.25, -0.95\n");
    scanf("%lf, %lf", &Alpha, &Beta);
    printf("You entered x(A) = %lf and x(B) = %lf\n", Alpha, Beta);
    printf("---------------------------------------------\n");
    printf("Please enter number of steps: ( < %d !)\n", MAX+1);
    scanf("%d", &N);
    if( N > MAX )
    {
    printf(" Number of steps must be less than %d\n",MAX+1);
    printf(" Terminating. Sorry\n");
    exit(0);
    }



    /* Do what subroutine CoeffMat in the book does */

    H = (B - A) / N;


    for( J = 0; J <=  N-2; J++) Vt[J] = A + H * (J + 1);

    for( J = 0; J <=  N-2; J++) Vb[J] = - pow(H,2) * rffunction(Vt[J]);

    Vb[0]   +=  Alpha * (1.0 + H / 2.0 * pffunction(Vt[0]));

    Vb[N-2] +=  Beta  * (1.0 - H / 2.0 * pffunction(Vt[N-2]));

    for( J = 0; J <=  N-2; J++) Vd[J] = 2.0 + pow(H,2) * qffunction(Vt[J]);

    for( J = 0; J <=  N-3; J++) Va[J] = - 1.0 - H * pffunction(Vt[J+1]) / 2.0;

    for( J = 0; J <=  N-3; J++) Vc[J] = - 1.0 + H * pffunction(Vt[J]) / 2.0;
     
    
    /* Do what subroutine TriMat in the book does */

    for ( K = 1; K <= N-2; K++)
    {
      Temp = Va[K-1] / Vd[K-1];
      Vd[K] -= Temp * Vc[K-1];
      Vb[K] -= Temp * Vb[K-1];
    }

    X0[N-2] = Vb[N-2] / Vd[N-2];

    for ( K = N-3; K >= 0; K--)
    {
       X0[K] = ( Vb[K] - Vc[K] * X0[K+1] ) / Vd[K];
    }

    for ( i = 0; i <= N-2; i++)
    {
      T[i+1] = Vt[i];
      X[i+1] = X0[i];
    }

    T[0] = A;
    T[N] = B;
    X[0] = Alpha;
    X[N] = Beta;

    /* Output */

    for ( J = 0; J <= N; J++)
    {
       printf("J = %d, T[J] = %lf, X[J] = %lf\n", J, T[J], X[J]);
    } 



}   /* End of main program */

